/*++
 *
 * Copyright (c) 2004-2006 Intel Corporation - All Rights Reserved
 * 
 * This software program is licensed subject to the BSD License, 
 * available at http://www.opensource.org/licenses/bsd-license.html
 * 
 * Source code modified by EnterpriseDB Corporation, to suit PostgreSQL's code.
 * 
 *
 --*/

#ifndef CRC_H_INCLUDED
#define CRC_H_INCLUDED

#include "postgres.h"
#include "pg_crc.h"

#define CRC32C_INIT_REFLECTED 0xFFFFFFFF
#define XOROT 0xFFFFFFFF

typedef enum {
	CRC_SB8_MODE_BEGIN = 0,
	CRC_SB8_MODE_CONT,
	CRC_SB8_MODE_END,
	CRC_SB8_MODE_BODY,
	CRC_SB8_MODE_ALIGN,

} CRC_SB8_MODE;


#ifdef __cplusplus
extern "C"
{
#endif	/* __cplusplus */


/* 
 * the following variables are used for counting cycles and bytes 
 */

extern uint32 crc_tableil8_o32[256];
extern uint32 crc_tableil8_o40[256];
extern uint32 crc_tableil8_o48[256];
extern uint32 crc_tableil8_o56[256];
extern uint32 crc_tableil8_o64[256];
extern uint32 crc_tableil8_o72[256];
extern uint32 crc_tableil8_o80[256];
extern uint32 crc_tableil8_o88[256];

/**
 *
 * Routine Description:
 *
 * Computes the CRC32c checksum for the specified buffer using the slicing by 8 
 * algorithm over 64 bit quantities.                      
 *
 * Arguments:
 *
 *		p_running_crc - pointer to the initial or final remainder value 
 *						used in CRC computations. It should be set to 
 *						non-NULL if the mode argument is equal to CONT or END
 *		p_buf - the packet buffer where crc computations are being performed
 *		length - the length of p_buf in bytes
 *		init_bytes - the number of initial bytes that need to be procesed before
 *					 aligning p_buf to multiples of 4 bytes
 *		mode - can be any of the following: BEGIN, CONT, END, BODY, ALIGN 
 *
 * Return value:
 *		
 *		The computed CRC32c value
 */

pg_crc32
crc32c_sb8_64_bit(
	pg_crc32* p_running_crc,
    const uint8*	p_buf,
    const uint32 length,
	CRC_SB8_MODE mode);

#ifdef __cplusplus
}
#endif	/* __cplusplus */

#endif	//CRC_H_INCLUDED

